#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

#################################################################################
# scsiaddgui - a gui for the scsiadd utility
# copyright (c) 2003 - 2006 Michael Lange <klappnase@8ung.at>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the #hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#################################################################################

import Tkinter
from ScrolledText import ScrolledText
import sys
import tkMessageBox
import commands
import fileinput
import os
import pwd
import stat
import locale
import gettext

Tkinter.wantobjects = 0
programPath = os.path.abspath(sys.path[0])

# check where to look for translations
localedir = '/usr/share/locale'
if programPath.startswith('/usr/local'):
    localedir = '/usr/local/share/locale'

gettext.install('scsiaddgui', localedir)

#####################################################################################################
#                                                                                                   #
#####################################################################################################

class Counter(Tkinter.Frame):
    def __init__(self, master, label=None, from_=0, to=100, increment=1, value=0, datatype='integer',
                 digits=0, width=5, command=None, **kw):
        Tkinter.Frame.__init__(self, master)
        
        self._command = command
        self.label = None
        if label:
            self.label = Tkinter.Label(self, text=label)
            self.label.pack(side='left')
            Tkinter.Frame(self, height=1, width=5).pack(side='left')
        self.spinbox = Tkinter.Spinbox(self, from_=from_, to=to, increment=increment, validate='all', invcmd=self.bell, width=width, **kw)
        self.spinbox.pack(side='left', fill='x', expand=1)
        if digits:
            format = '%' + '0.%df' % digits
            self.spinbox.config(format=format)
        
        self._datatype = datatype
        self._oldvalue = value
        self.set(value)
        
        vcmd = (self.register(self._validate), '%s', '%P')
        self.spinbox.config(vcmd=vcmd)

    def _validate(self, old, new):
        if (old != '') and (float(self.spinbox['from']) <= float(old) <= float(self.spinbox['to'])):
            self._oldvalue = old
        if new == '':
            return 1
        try:
            if self._datatype == 'integer':
                x = int(new)
            elif self._datatype == 'float':
                x = float(new)
            if 0 <= x <= float(self.spinbox['to']):
                if self._command and (new != self._oldvalue):
                    self._command(new)
                return 1
            return 0
        except ValueError:
            return 0
    
    def get(self):
        val = self.spinbox.get()
        if val == '':
            self.set(self._oldvalue)
        # now see if the current value is maller than the allowed minimum
        # this is allowed for editing but not for retieving the value
        elif self._datatype == 'integer':
            if int(val) < float(self.spinbox['from']):
                self.set(self._oldvalue)
        elif self._datatype == 'float':
            if float(val) < float(self.spinbox['from']):
                self.set(self._oldvalue)
        if self._datatype == 'integer':
            return int(self.spinbox.get())
        elif self._datatype == 'float':
            return float(self.spinbox.get())
    
    def set(self, value=None):
        return self.tk.call(self.spinbox._w, 'set', value)
    
    def disable(self):
        self.spinbox.config(state='disabled')
    
    def enable(self):
        self.spinbox.config(state='normal')
    
    def focus_set(self):
        self.spinbox.focus_set()
    focus = focus_set
    
    def focus_get(self):
        return self.spinbox.focus_get()
    
    def bind(self, sequence, callback, *args):
        return self.spinbox.bind(sequence, callback, *args)

#####################################################################################################
#                                                                                                   #
#####################################################################################################

class ToolTip:
    def __init__(self, master, text='Your text here', delay=1500, **opts):
        self.master = master
        self._opts = {'anchor':'center', 'bd':1, 'bg':'lightyellow', 'delay':delay, 'fg':'black',\
                      'follow_mouse':0, 'font':None, 'justify':'left', 'padx':4, 'pady':2,\
                      'relief':'solid', 'state':'normal', 'text':text, 'textvariable':None,\
                      'width':0, 'wraplength':150}
        self.configure(**opts)
        self._tipwindow = None
        self._id = None
        self._id1 = self.master.bind("<Enter>", self.enter, '+')
        self._id2 = self.master.bind("<Leave>", self.leave, '+')
        self._id3 = self.master.bind("<ButtonPress>", self.leave, '+')
        self._follow_mouse = 0
        if self._opts['follow_mouse']:
            self._id4 = self.master.bind("<Motion>", self.motion, '+')
            self._follow_mouse = 1
    
    def configure(self, **opts):
        for key in opts:
            if self._opts.has_key(key):
                self._opts[key] = opts[key]
            else:
                KeyError = 'KeyError: Unknown option: "%s"' %key
                raise KeyError
    
    ##----these methods can be used if the master widget has already bindings---------------##
    ##----for "<Enter>", "<Leave>" or "<ButtonPress>" events to keep the tooltip working:---##
    
    def enter(self, event=None):
        self._schedule()
        
    def leave(self, event=None):
        self._unschedule()
        self._hide()
    
    def motion(self, event=None):
        if self._tipwindow and self._follow_mouse:
            x, y = self.coords()
            self._tipwindow.wm_geometry("+%d+%d" % (x, y))
    
    ##------the methods that do the work:-------------------------------##
    
    def _schedule(self):
        self._unschedule()
        if self._opts['state'] == 'disabled':
            return
        self._id = self.master.after(self._opts['delay'], self._show)

    def _unschedule(self):
        id = self._id
        self._id = None
        if id:
            self.master.after_cancel(id)

    def _show(self):
        if self._opts['state'] == 'disabled':
            self._unschedule()
            return
        if not self._tipwindow:
            self._tipwindow = tw = Tkinter.Toplevel(self.master)
            # hide the window until we know the geometry
            tw.withdraw()
            tw.wm_overrideredirect(1)
            self.create_contents()
            tw.update_idletasks()
            x, y = self.coords()
            tw.wm_geometry("+%d+%d" % (x, y))
            tw.deiconify()
    
    def _hide(self):
        tw = self._tipwindow
        self._tipwindow = None
        if tw:
            tw.destroy()
                
    ##----these methods might be overridden in derived classes:-------------##
    
    def coords(self):
        # The tip window must be completely outside the master widget;
        # otherwise when the mouse enters the tip window we get
        # a leave event and it disappears, and then we get an enter
        # event and it reappears, and so on forever :-(
        # or we take care that the mouse pointer is always outside the tipwindow :-)
        tw = self._tipwindow
        twx, twy = tw.winfo_reqwidth(), tw.winfo_reqheight()
        w, h = tw.winfo_screenwidth(), tw.winfo_screenheight()
        # calculate the y coordinate:
        if self._follow_mouse:
            y = tw.winfo_pointery() + 20
            # make sure the tipwindow is never outside the screen:
            if y + twy > h:
                y = y - twy - 30
        else:
            y = self.master.winfo_rooty() + self.master.winfo_height() + 3
            if y + twy > h:
                y = self.master.winfo_rooty() - twy - 3
        # we can use the same x coord in both cases:
        x = tw.winfo_pointerx() - twx / 2
        if x < 0:
            x = 0
        elif x + twx > w:
            x = w - twx
        return x, y

    def create_contents(self):
        opts = self._opts.copy()
        for opt in ('delay', 'follow_mouse', 'state'):
            del opts[opt]
        label = Tkinter.Label(self._tipwindow, **opts)
        label.pack()

#####################################################################################################
#                                                                                                   #
#####################################################################################################

class ReadonlyScrolledText(ScrolledText):
    def __init__(self, master, **kw):
        ScrolledText.__init__(self, master, cursor='', wrap='word', padx=0, pady=4, takefocus=0, **kw)
        self.frame.configure(relief='groove', bd=2, highlightthickness=1, highlightbackground=self.frame['bg'])
        self.vbar.configure(highlightthickness=0)
        self.vbar.bind('<FocusIn>', lambda event: self.frame.configure(highlightbackground=self.frame['highlightcolor']))
        self.vbar.bind('<FocusOut>', lambda event: self.frame.configure(highlightbackground=self.frame['bg']))
    def focus(self):
        self.vbar.focus_set()

#####################################################################################################
#                                                                                                   #
#####################################################################################################

class HelpDialog(Tkinter.Toplevel):
    def __init__(self, master, **kw):
        Tkinter.Toplevel.__init__(self, master, **kw)
        self.protocol('WM_DELETE_WINDOW', self.withdraw)
        self.title(_('ScsiaddGui - Help'))
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)
        self.master = master
        
        self.text = ReadonlyScrolledText(self, bg="lightyellow")
        self.text.grid(row=0, column=0, sticky='news', padx=3, pady=3)
        helpfile = self.get_helpfile()
        for line in fileinput.input(helpfile):
            self.text.insert('end', line)
        self.text.config(state='disabled')
        Tkinter.Button(self, text=_('Close'), command=self.withdraw).grid(row=1, column=0, pady=5)
    
    def get_helpfile(self):
        try:
            lang = locale.getdefaultlocale()[0][:2]
        except TypeError:
            # no language found, fall back to english
            lang = 'en'
        helpfile = os.path.join(programPath, 'help_'+lang)
        if not (os.path.exists(helpfile) and os.access(helpfile, os.R_OK)):
            return os.path.join(programPath, 'help_en')
        return helpfile
        
#####################################################################################################
#                                                                                                   #
#####################################################################################################

class OptConf:
    def __init__(self):
        self.opts = {'host':0, 'chan':0, 'i_d':0, 'lun':0}
        self.conffile = os.path.join(os.getenv('HOME'), '.scsiaddgui')

    def read_conffile(self):
        try:
            for line in fileinput.input(self.conffile):
                try:
                    l = line.split('=')
                    if self.opts.has_key(l[0]):
                        self.opts[l[0]] = int(l[1])
                except:
                    pass
        except:
            pass

    def write_conffile(self):
        f = open(self.conffile, 'w')
        for key in self.opts:
            f.write(key + '=' + str(self.opts[key]) + '\n')
        f.close()
        
#####################################################################################################
#                                                                                                   #
#####################################################################################################

class Mainwindow(Tkinter.Frame):
    def __init__(self, master, **kw):
        Tkinter.Frame.__init__(self, master, **kw)
        self.grid_rowconfigure(1, weight=1)
        for i in range(0, 4):
            self.grid_columnconfigure(i, weight=1)

        self.master = master
        self.help_dialog = None
        self.conf = OptConf()
        self.conf.read_conffile()
        
        Tkinter.Label(self, text=_('Attached devices:')).grid(row=0, column=0, columnspan=4, pady=4)
        self.display = ReadonlyScrolledText(self, width=70, height=15, state='disabled')
        self.display.grid(row=1, column=0, columnspan=4, sticky='news', padx=4)
        self.display.tag_config('white', lmargin1=5, rmargin=5)
        self.display.tag_config('blue', background='LightBlue1', lmargin1=5, rmargin=5)
        self.display.tag_raise('sel')
        
        self.host = Counter(self, from_=0, to=15, label=_('Host:'), value=self.conf.opts['host'])
        self.host.grid(row=2, column=0, pady=4)
        self.chan = Counter(self, from_=0, to=15, label=_('Channel:'), value=self.conf.opts['chan'])
        self.chan.grid(row=2, column=1, pady=4)
        self.i_d = Counter(self, from_=0, to=15, label=_('Id:'), value=self.conf.opts['i_d'])
        self.i_d.grid(row=2, column=2, pady=4)
        self.lun = Counter(self, from_=0, to=255, label=_('Lun:'), value=self.conf.opts['lun'])
        self.lun.grid(row=2, column=3, pady=4)
        
        self.scanicon = Tkinter.PhotoImage(data='R0lGODlhEAAQAPeCAAAAAAAMFg8XFQAsTgAvUAAxWBNEa2I5AXVCAHdFAHhHAH5MAVhmdBNGgRdRiR'
                                           'lShx1ZjjRjhi9noTtwozhsxEt+rFSHnFKKtmKTvl+K8HGfyHGf0HiV9mqg0G6g23Gg0Xel0Hml0H+s2IVLD'
                                           'IdPCYRPDYlRDJJbGYt/ALNwDcBxDcCDMMiMPNygJsOXSOO8deW/e+bAe/zlavX8c4Wv2Ymm9Yqi/Yil/Y+6'
                                           '5JC75ZGx9ZGw9pOy/pa0/5G7+Zi595u8+I/A7ZfD9J/A+KTK4qHD+KPC+qLE+KTD/6XI+abI+ajM+qvO+qv'
                                           'Q+qjU/q/S+67S/K/S/bHV8rfX8rDT+7PS/7TZ/Lbb/7jb/7ze/7/d/Lrj/dvAleLGkuHGlMHd/MPh/cPn/8'
                                           'Th/sXo/8rl/c3l/dTp/t3v/t/w/tz0/+Lx/uTz/+bz/ubz/+T3/+X2/+f0/+r1/+r2/uv2/+v3/+73/+n6/'
                                           '+z4/+/8//H5//T6//f7//b9//f8//f+//f///v8//v//////wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'
                                           'AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAIIALAAAAAAQABAAAAizAAUJ/CAihAYMAhMqFJR'
                                           'Dihs+f+4QibBQII40gQDpoQPHzhYDCzdM8dMnz5w1Z8iEubCQBp49deSwQVPGChYnAxSCeBOnjRozWqg0iSKE'
                                           'gMIOY8CI+fJkSZIiRnwUWBgkCxQmSob80NFDQoCFFa5UOQJkRw0bGSygqDjBAxIeNzhQYDBDBoKKgiA8cNBAg'
                                           'IoWXGIkwLtwxIouMBQQVmhihZcXCxYnJMHCRQrJCUucOIB5YUAAOw==')
        self.addicon = Tkinter.PhotoImage(data='R0lGODlhEAAQAKU6AAAAAAACAAAEAAAQAAASAAEbAQEhAgEjAgEnAgIuAwI0BAI6BgRICgZMCQRSDQV'
                                          'VDwdXDQVdEQZgEwhgEAZiEwdtGAhwGQl5Hg99HAyOLg6SMA+bNxOrRROvShW1USC0Rhe5VBe7WRnHYxvJaRz'
                                          'KaijciS/bjDHbjT/XijjakEDYj0LYk0fXklPdmlbgnVjgolvhoGvprHDqr3bsrofxvojyv5L0xZj2yav508j'
                                          '84////////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQBCgA/ACwAAAAAEAAQAAA'
                                          'Ga8CfcEj8AQDFpDCAEQGURMBHZgBAhxOcBkD0JIqNmeoAIDJykSEAVdsAkqZb5QeA0E4LQLIAc1EKMS0cAlA'
                                          'SLysZNyUPAFACIyw2KyAEVz8KKSQhFgCWPxUdFwgAngMOCACeQgFHqkIArp5BADs=')
        self.removeicon = Tkinter.PhotoImage(data='R0lGODlhEAAQAMZlAAEBAQICAgUFBQgFBQkFBBIFAgkJCQ8JBw4LCRsHBA0NDRgJBhwJBSAIBCAL'
                                             'CCMKBh0PDCQNCSwLBSwOCSYRDSYRDi8OByUSDysTDiwUDy8VEC0XFDEXEjIYEzAZFDsXETQaFTgZFDwYEk'
                                             'EXEDkcF0kWDE4WDT4eGEwbE0IgGU4dFFchGVwlG4ImF5IsG5EvHpIvHpYuHJQwHpkvHZYxHpgxHqE1IaI6'
                                             'Jq02ILA3ILI4IbQ5IrY5IrQ6I7c5Irk6Iro6Irk7JLo7Irs7IrY9Jrw7Ir07Ir47Ir87Ir08JLg+J8A7Is'
                                             'I7Ir09Jb0/KL4/J61GMsJBKcNMNsRNNsZNN8hON7RbSs5SOslWQdBfSdNjTs5mU8FuXsptW8xyYc1yYdZv'
                                             'W9lwW8t4aNF/cN2Nff/////////////////////////////////////////////////////////////////'
                                             '//////////////////////////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQBCg'
                                             'B/ACwAAAAAEAAQAAAHo4B/goOEhYMpJAqGfwECgx1kWSqGG1xQBIInWlhUKIQeYVExBYIIUktIUyyCEGBBL'
                                             'iaEGlJHRzofAl1KLRKGGFJGQj5WVzMNi38iUkBAVTwWAcgBJVtJQEM7GcgJY09EPkVGORiGIWJOMAYfUrU9F'
                                             '4MAK14/NA6CI1NISzcDgg9fTWpEIISCChMcFAQdeCFjQSEFKqrYqDBoAoNFCkBwQMbxTyAAOw==')
        
        bframe = Tkinter.Frame(self)
        bframe.grid(row=3, column=0, columnspan=4, pady=4, padx=8, sticky='ew')
        scanbutton = Tkinter.Button(bframe, text=_('Scan for devices'), image=self.scanicon, compound='left', command=self.scan)
        scanbutton.grid(row=0, column=0, padx=8, sticky='ew')
        addbutton = Tkinter.Button(bframe, text=_('Add device'), image=self.addicon, compound='left', command=self.add)
        addbutton.grid(row=0, column=1, padx=8, sticky='ew')
        removebutton = Tkinter.Button(bframe, text=_('Remove device'), image=self.removeicon, compound='left', command=self.remove)
        removebutton.grid(row=0, column=2, padx=8, sticky='ew')
        # make all three buttons have the same width
        self.update_idletasks()
        max = 0
        for b in (scanbutton, addbutton, removebutton):
            if b.winfo_reqwidth() + 16 > max:
                max = b.winfo_reqwidth() + 16
        for c in range(3):
            bframe.grid_columnconfigure(c, weight=1, minsize=max)
        master.minsize(width=self.winfo_reqwidth(), height=240)
        
        ToolTip(self.host, _('Select the SCSI adapter card; 0 means the first card, 1 the second and so on'))
        ToolTip(self.chan, _('Select the bus-no. of your adapter card (most cards only support bus 0)'))
        ToolTip(self.i_d, _('Select the ID-no. of your device'))
        ToolTip(self.lun, _("Select the Logical Unit Number if you have a device that supports multiple LUN's"))
        ToolTip(scanbutton, _('Try to find all devices connected to the selected SCSI Bus/Channel'))
        ToolTip(addbutton, _('Add the selected device to the system.'))
        ToolTip(removebutton, _('Remove the selected device from the system'))

        self.locked = 0
        self.scsiadd = self.find_scsiadd()
        # now find out if we are allowed to change the hardware configuration
        use_scsiadd_ok = 1
        if self.scsiadd in ('',  'permission denied'):
            use_scsiadd_ok = 0
        if os.getuid():
            # we are not root, see if scsiadd has super user privileges
            oct_mode = oct(stat.S_IMODE(os.stat(self.scsiadd)[0]))
            # now we should have something like "0755" or "4755"
            if int(oct_mode[0]) < 4:
                use_scsiadd_ok = 0
        
        if not use_scsiadd_ok:
            # if we don't have permission to change the hardware configuration, diable the widgets
            for w in (self.host.spinbox, self.chan.spinbox, self.i_d.spinbox, self.lun.spinbox, scanbutton, addbutton, removebutton):
                w.config(state='disabled')
        
        if not self.scsiadd:
            tkMessageBox.showerror(title=_('ScsiaddGui'),
                                   message=_('Cannot find "scsiadd". Please check your installation.'))
        else:
            self.update()
    
    def find_scsiadd(self):
        # check if scsiadd is available
        paths = os.getenv('PATH').split(':')
        for p in ('/sbin', '/usr/sbin', '/usr/local/sbin'):
            if not p in paths:
                paths.append(p)
        for p in paths:
            fname = os.path.join(p, 'scsiadd')
            if os.path.isfile(fname):
                if os.access(fname, os.X_OK):
                    return fname
                else:
                    return 'permission denied'
        return ''
        
    def run(self, action):
        while self.locked:
            if action == 'print':
                return
        self.locked = 1
        if action == 'print':
            cmd = '%s -p' % self.scsiadd
        elif action == 'scan':
            oldcursor = self.master.cget('cursor')
            self.master.configure(cursor='watch')
            self.master.update_idletasks()
            cmd = '%s -s %d %d' % (self.scsiadd, self.host.get(), self.chan.get())
        elif action == 'add':
            cmd = '%s -a %d %d %d %d' % (self.scsiadd, self.host.get(), self.chan.get(), self.i_d.get(), self.lun.get())
        else:
            cmd = '%s -r %d %d %d %d' % (self.scsiadd, self.host.get(), self.chan.get(), self.i_d.get(), self.lun.get())
        self.update_display(commands.getoutput(cmd))
        if action == 'scan':
            self.master.configure(cursor=oldcursor)
        self.locked = 0
                
    def format_msg(self, msg):
        if msg[-1] != '\n':
            msg = msg + '\n'
        msg = msg.replace('Vendor:', _('Vendor:'))
        msg = msg.replace('Model:', _('Model:'))
        msg = msg.replace('Type:', _('Type:'))
        msg = msg.replace('Host:', _('Host:'))
        msg = msg.replace('Channel:', _('Channel:'))
        msg = msg.replace('Id:', _('Id:'))
        msg = msg.replace('Lun:', _('Lun:'))
        return msg
        
    def toggle_tag(self, tag):
        if tag == 'white':
            return 'blue'
        return 'white'
    
    def update_display(self, message):
        # split the output message into a list of devices
        # quick'n'dirty: use a german special character as separator
        message = message.replace('Host:', '\xdfHost:')
        if '\xdf' in message:
            # one or more devices were found
            dlist = message.split('\xdf')[1:]
        else:
            # no devices found
            dlist = [_('No SCSI devices found.')]
        self.display['state'] = 'normal'
        self.display.delete(0.0, 'end')
        tag = 'white'
        for item in dlist:
            msg = self.format_msg(item)
            self.display.insert('end', msg, tag)
            tag = self.toggle_tag(tag)
        self.display['state'] = 'disabled'

    def show_help(self):
        if self.help_dialog:
            self.help_dialog.tkraise()
            self.help_dialog.deiconify()
        else:
            self.help_dialog = HelpDialog(self, relief='ridge', bd=3)
        self.help_dialog.text.focus()
    
    def add(self):
        self.run('add')
    
    def remove(self):
        self.run('remove')
    
    def scan(self):
        self.run('scan')
    
    def update(self):
        self.run('print')
        self.after(5000, self.update)

    def close(self):
        while self.locked:
            pass
        self.conf.opts['host'] = self.host.get()
        self.conf.opts['chan'] = self.chan.get()
        self.conf.opts['i_d'] = self.i_d.get()
        self.conf.opts['lun'] = self.lun.get()
        self.conf.write_conffile()
        self.master.quit()
    
#####################################################################################################
#                                                                                                   #
#####################################################################################################

def infobox():
    tkMessageBox.showinfo(title=_('About ScsiaddGui'),
                          message=_('ScsiaddGui - by Michael Lange\nContact: <klappnase@8ung.at>'))

optionDB = {
'*font' : 'Helvetica -12',
'*Spinbox*background' : 'white',
'*selectBackground' : 'blue4',
'*selectForeground' : 'white',
'*inactiveSelectBackground' : 'blue4',
'*Scrollbar*Width' : '12',
'*Menu*tearOff' : '0',
'*Menu*relief' : 'raised',
'*Menu*borderWidth' : '2',
'*Text*background' : 'white'
}

def main():
    root = Tkinter.Tk()
    root.withdraw()
    for opt in optionDB:
        root.option_add(opt, optionDB[opt])
    root.title(string='ScsiaddGui (%s)' % pwd.getpwuid(os.getuid())[0])
    root.bind_class('Button', '<Return>', lambda event :root.event_generate('<space>'))
    mw = Mainwindow(root, relief='ridge', bd=3)
    mw.pack(fill='both', expand=1)
    root.protocol('WM_DELETE_WINDOW', mw.close)
    menubar = Tkinter.Menu(root)
    progm = Tkinter.Menu(menubar)
    progm.add_command(label=_('Quit'), command=mw.close)
    menubar.add_cascade(label=_('Program'), underline=0, menu=progm)
    helpm = Tkinter.Menu(menubar)
    helpm.add_command(label=_('Help...'), command=mw.show_help)
    helpm.add_command(label=_('About...'), command=infobox)
    menubar.add_cascade(label=_('Help'), underline=0, menu=helpm)
    root.config(menu=menubar)
    root.update_idletasks()
    root.deiconify()
    root.mainloop()
    root.destroy()
        
if __name__ == "__main__":
    main()
